@@ -53,11 +53,6 @@ class EncodedVideoPyAV(EncodedVideo):
 
         video_duration = video_stream.duration
 
-        # If video duration is None we can set it to the container duration which may
-        # be available.
-        if video_duration is None:
-            video_duration = self._container.duration
-
         # Retrieve audio header information if available.
         audio_duration = None
         self._has_audio = None
