@@ -171,6 +171,94 @@ class Normalize(torchvision.transforms.Normalize):
         return vid
 
 
+class ConvertUint8ToFloat(torch.nn.Module):
+    """
+    Converts a video from dtype uint8 to dtype float32.
+    """
+
+    def __init__(self):
+        super().__init__()
+        self.convert_func = torchvision.transforms.ConvertImageDtype(torch.float32)
+
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        """
+        Args:
+            x (torch.Tensor): video tensor with shape (C, T, H, W).
+        """
+        assert x.dtype == torch.uint8, "image must have dtype torch.uint8"
+        return self.convert_func(x)
+
+
+class RandomResizedCrop(torch.nn.Module):
+    """
+    ``nn.Module`` wrapper for ``pytorchvideo.transforms.functional.random_resized_crop``.
+    """
+
+    def __init__(
+        self,
+        target_height: int,
+        target_width: int,
+        scale: Tuple[float, float],
+        aspect_ratio: Tuple[float, float],
+        shift: bool = False,
+        log_uniform_ratio: bool = True,
+        interpolation: str = "bilinear",
+        num_tries: int = 10,
+    ) -> None:
+
+        super().__init__()
+        self._target_height = target_height
+        self._target_width = target_width
+        self._scale = scale
+        self._aspect_ratio = aspect_ratio
+        self._shift = shift
+        self._log_uniform_ratio = log_uniform_ratio
+        self._interpolation = interpolation
+        self._num_tries = num_tries
+
+    def __call__(self, x: torch.Tensor) -> torch.Tensor:
+        """
+        Args:
+            x (torch.Tensor): Input video tensor with shape (C, T, H, W).
+        """
+        return pytorchvideo.transforms.functional.random_resized_crop(
+            x,
+            self._target_height,
+            self._target_width,
+            self._scale,
+            self._aspect_ratio,
+            self._shift,
+            self._log_uniform_ratio,
+            self._interpolation,
+            self._num_tries,
+        )
+
+
+class Permute(torch.nn.Module):
+    """
+    Permutes the dimensions of a video.
+    """
+
+    def __init__(self, dims: Tuple[int]):
+        """
+        Args:
+            dims (Tuple[int]): The desired ordering of dimensions.
+        """
+        assert (
+            (d in dims) for d in range(len(dims))
+        ), "dims must contain every dimension (0, 1, 2, ...)"
+
+        super().__init__()
+        self._dims = dims
+
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        """
+        Args:
+            x (torch.Tensor): video tensor whose dimensions are to be permuted.
+        """
+        return x.permute(*self._dims)
+
+
 class OpSampler(torch.nn.Module):
     """
     Given a list of transforms with weights, OpSampler applies weighted sampling to
@@ -182,6 +270,7 @@ class OpSampler(torch.nn.Module):
         transforms_list: List[Callable],
         transforms_prob: Optional[List[float]] = None,
         num_sample_op: int = 1,
+        randomly_sample_depth: bool = False,
         replacement: bool = False,
     ):
         """
@@ -193,6 +282,8 @@ class OpSampler(torch.nn.Module):
                 uniform distribution over all transforms. They do not need to sum up to one
                 but weights need to be positive.
             num_sample_op (int): Number of transforms to sample and apply to input.
+            randomly_sample_depth (bool): If randomly_sample_depth is True, then uniformly
+                sample the number of transforms to apply, between 1 and num_sample_op.
             replacement (bool): If replacement is True, transforms are drawn with replacement.
         """
         super().__init__()
@@ -218,6 +309,7 @@ class OpSampler(torch.nn.Module):
             else [1] * len(transforms_list)
         )
         self.num_sample_op = num_sample_op
+        self.randomly_sample_depth = randomly_sample_depth
         self.replacement = replacement
 
     def forward(self, x: torch.Tensor) -> torch.Tensor:
@@ -225,11 +317,35 @@ class OpSampler(torch.nn.Module):
         Args:
             x (torch.Tensor): Input tensor.
         """
+        depth = (
+            torch.randint(1, self.num_sample_op + 1, (1,)).item()
+            if self.randomly_sample_depth
+            else self.num_sample_op
+        )
         index_list = torch.multinomial(
-            self.transforms_prob, self.num_sample_op, replacement=self.replacement
+            self.transforms_prob, depth, replacement=self.replacement
         )
 
         for index in index_list:
             x = self.transforms_list[index](x)
 
         return x
+
+
+class Div255(torch.nn.Module):
+    """
+    ``nn.Module`` wrapper for ``pytorchvideo.transforms.functional.div_255``.
+    """
+
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        """
+        Scale clip frames from [0, 255] to [0, 1].
+        Args:
+            x (Tensor): A tensor of the clip's RGB frames with shape:
+                (C, T, H, W).
+        Returns:
+            x (Tensor): Scaled tensor by dividing 255.
+        """
+        return torchvision.transforms.Lambda(
+            pytorchvideo.transforms.functional.div_255
+        )(x)
